/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;

import java.util.Calendar;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import no.ntnu.es.bb.schema.Assessment;
import no.ntnu.es.bb.schema.AssessmentDAO;
import no.ntnu.es.bb.schema.rubric.*;
import no.ntnu.es.bb.schema.rubric.RubricColumnDAO;
import no.ntnu.es.bb.schema.rubric.RubricDAO;
import no.ntnu.es.bb.schema.rubric.RubricRowDAO;
import blackboard.base.FormattedText;
import blackboard.data.ExtendedData;
import blackboard.data.ReceiptOptions;
import blackboard.data.ValidationException;
import blackboard.data.content.Content;
import blackboard.data.course.Course;
import blackboard.data.course.Group;
import blackboard.data.user.User;
import blackboard.persist.DataType;
import blackboard.persist.Id;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.PersistenceException;
import blackboard.persist.PersistenceRuntimeException;
import blackboard.persist.content.ContentDbLoader;
import blackboard.persist.content.ContentDbPersister;
import blackboard.persist.course.GroupDbLoader;
import blackboard.persist.user.UserDbLoader;
import blackboard.platform.context.Context;
import blackboard.platform.context.ContextManager;
import blackboard.platform.context.ContextManagerFactory;
import blackboard.platform.filesystem.FileSystemException;
import blackboard.platform.filesystem.manager.CourseFileManager;
import blackboard.platform.gradebook2.GradableItem;
import blackboard.platform.gradebook2.GradableItemManager;
import blackboard.platform.gradebook2.GradeManager;
import blackboard.platform.gradebook2.GradebookManagerFactory;
import blackboard.platform.plugin.PlugInUtil;
import blackboard.platform.security.authentication.BbSecurityException;
import blackboard.platform.servlet.InlineReceiptUtil;
import blackboard.platform.spring.beans.annotations.UserAuthorization;
import blackboard.servlet.data.MultiSelectBean;
import blackboard.servlet.util.DatePickerUtil;

// Denote this class as a Spring Controller
@Controller
public class AssessmentController {
	private static final Logger LOGGER = Logger.getLogger(AssessmentController.class.getName());
	private final String ASSESSMENT_ID = "assessmentID";
	
	// Autowire -- a.k.a. auto instantiate -- the DAOs for use in the controller
	@Autowired
	private AssessmentDAO assess_dao;
	
	@Autowired
	private RubricDAO rubric_dao;
	
	@Autowired
	private RubricColumnDAO rubric_column_dao;
	
	@Autowired
	private RubricRowDAO rubric_row_dao;
	
	@Autowired
	private RubricCellDAO rubric_cell_dao;
	
	@Autowired
	private StudentReviewDAO student_review_dao;
	
	@Autowired
  	private UserDbLoader _userLoader;
	
	@Autowired
	private GroupDbLoader _groupLoader;


	// Any time Spring receives a request for the URL containing /assessment/create, execute this method. 
	// The method itself returns a ModelAndView. 
	// Spring uses this object to locate the appropriate jsp file and pass any variables that has been set.
	// This method is ran when the user creates a new assessment
	@UserAuthorization("system.admin.VIEW")
	@RequestMapping("/assessment/create")
	public ModelAndView create(HttpServletRequest request, HttpServletResponse response) {
		
		// Create a new ModelAndView Object where the constructor takes a string that
		// corresponds to the name of the jsp file the object should be routed to
		ModelAndView mv = new ModelAndView("assessment/create");
		
		//Course ID could also be added through @RequestParam("course_id") String course_id 
		ContextManager contextManager = ContextManagerFactory.getInstance();
		Context ctx = contextManager.getContext();

		Course crs = ctx.getCourse();
		try {
			List<User> userList = _userLoader.loadByCourseId(crs.getId());
			mv.addObject("numberOfStudents", 0);
			
			if(userList != null){
				int number = 0;
				for(User tempUser : userList){
					if(tempUser.getInstitutionRole() == User.InstRole.STUDENT){
						number++;
					}
				}
				LOGGER.debug("numberOfStudentsAdded: " + number);
				mv.addObject("numberOfStudents", number);
			}
		
			
			/**
			 * Adding groups to Group Submission for assignment
			 */
			List<Group> groupList = _groupLoader.loadByCourseId(crs.getId());
			List<MultiSelectBean> multiSelectOptions = new ArrayList<MultiSelectBean>();
			for (Iterator iterator = groupList.iterator(); iterator.hasNext();) {
				Group group = (Group) iterator.next();
				MultiSelectBean selectOption1 = new MultiSelectBean(group.getTitle(),group.getId().toString());
				multiSelectOptions.add(selectOption1);
			}
			mv.addObject("usersInCourse", userList);
			mv.addObject("groupsInCourse", multiSelectOptions);
		} catch (KeyNotFoundException e) {
			e.printStackTrace();
		} catch (PersistenceException e) {
			e.printStackTrace();
		}
		
		LOGGER.debug("assessment/create is now loaded!");

		// Returning the ModelAndView tells spring to find the JSP and set these
		// objects in the context for the request.
		return mv;
	}

	// This method is ran when the user edits an assessment
	@UserAuthorization("system.admin.VIEW")
	@RequestMapping("/assessment/edit")
	public ModelAndView edit(HttpServletRequest request, HttpServletResponse response,
			@RequestParam("content_id") String content_id, 
			@RequestParam("course_id") String course_id 
			) {
	
		ModelAndView mv = new ModelAndView("assessment/edit");


		//In order to get the asssessment ID (which is not sent through the URI) we get it from Content extra data.
		ContentDbLoader loader;
		Id contentID;
		Content courseDoc = null;
		try {
			loader = ContentDbLoader.Default.getInstance();
			contentID = Id.generateId( Content.DATA_TYPE, content_id );
			courseDoc  = loader.loadById(contentID);
		} catch (PersistenceException e) {
			LOGGER.debug("Failed to load Content in /edit " );
			e.printStackTrace();
		}
		 

		ExtendedData courseEx = courseDoc.getExtendedData();
		String assessment_id = courseEx.getValue(ASSESSMENT_ID);
		LOGGER.debug("CourseDoc: "+ courseDoc+ " ExtendedData: "+ courseEx+ " assessmentID: "+ assessment_id );
		Assessment assessment = assess_dao.loadById(Integer.parseInt(assessment_id));
		
		// get the string name from the database, convert it to FormattedText and store it in the name variable.
		String assessment_name = assessment.getName();

		FormattedText assessment_instructions = FormattedText.toFormattedText(assessment.getInstructions());
		FormattedText evaluation_instructions = FormattedText.toFormattedText(assessment.getEvaluationInstructions());

		
		int nr_of_submissions = assessment.getNrOfSubmissions();
		
		int review_of_review_max_score = assessment.getReviewOfReviewMaxScore();
		
		Calendar startSubDate = assessment.getSubmissionDateStart();
		Calendar endSubDate = assessment.getSubmissionDateEnd();
		Calendar startPeerDate = assessment.getPeerDateStart();
		Calendar endPeerDate = assessment.getPeerDateEnd();
		
		//Sets the selected groups for the assessment
		List<MultiSelectBean> multiSelectOptionsForGroups = new ArrayList<MultiSelectBean>();
		String[] submissionGroups = null;
		if(assessment.getSubmissionGroups() != null){
			submissionGroups = assessment.getSubmissionGroups().split("(?<=}),(?=P)");
		}else {
			submissionGroups = new String[1];
			submissionGroups[0] = "";
		}
		
		LOGGER.debug("submissionGroups: " + submissionGroups.toString());
		if (!submissionGroups[0].equals("")) {
			String subString = "";
			int index = 0;
			for (int i = 0; i < submissionGroups.length; i++) {
				if(i == submissionGroups.length-1){	
					for (int j = submissionGroups[i].toString().length()-1; j > 0; j--) {
						if(submissionGroups[i].toString().charAt(j) != ','){
							subString = submissionGroups[i].toString().substring(0, j+1);
							index = i;
							LOGGER.debug("INSIDE! i:" + index);
							break;
						}
					}
					
					
				}
				LOGGER.debug("Split String3: " + submissionGroups[i]);
			}
			LOGGER.debug("String on index( "+index+" : " + submissionGroups[index] + " Substring: "+subString);
			submissionGroups[index] = subString;
			LOGGER.debug("Split String X3: " + submissionGroups[submissionGroups.length-1]);
			for (String string : submissionGroups) {
				String groupName = "failed to get group";
				try {
					groupName = _groupLoader.loadById(Id.generateId(Group.DATA_TYPE, string)).getTitle();
				} catch (KeyNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (PersistenceException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				MultiSelectBean selectOption2 = new MultiSelectBean(groupName, string);
				multiSelectOptionsForGroups.add(selectOption2);
			}
			mv.addObject("groupsSetForSubmission", multiSelectOptionsForGroups);
		}
		//Course ID could also be added through @RequestParam("course_id") String course_id 
		ContextManager contextManager = ContextManagerFactory.getInstance();
		Context ctx = contextManager.getContext();

		Course crs = ctx.getCourse();
		mv.addObject("numberOfStudents", 0);
		try {
			List<User> userList = _userLoader.loadByCourseId(crs.getId());
			if(userList != null){
				int number = 0;
				for(User tempUser : userList){
					if(tempUser.getInstitutionRole() == User.InstRole.STUDENT){
						number++;
					}
				}
				LOGGER.debug("numberOfStudentsAdded: " + number);
				mv.addObject("numberOfStudents", number);
			}
		} catch (PersistenceException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
		
		
		/**
		 * Adding groups to Group Submission for assignment
		 */
		try {
			List<Group> groupList = _groupLoader.loadByCourseId(crs.getId());
			List<MultiSelectBean> multiSelectOptions = new ArrayList<MultiSelectBean>();
			for (Iterator iterator = groupList.iterator(); iterator.hasNext();) {
				Group group = (Group) iterator.next();
				boolean idNotUsed = true;
				for (String groupId : submissionGroups) {
					if(groupId.equals(group.getId().toString())){
						idNotUsed = false;
					}
				}
				
				if(idNotUsed){
					MultiSelectBean selectOption1 = new MultiSelectBean(group.getTitle(),group.getId().toString());
					multiSelectOptions.add(selectOption1);
				}
			}
		
			mv.addObject("groupsInCourse", multiSelectOptions);
		} catch (KeyNotFoundException e) {
			e.printStackTrace();
		} catch (PersistenceException e) {
			e.printStackTrace();
		}
		
		// Sets correct radio button values
		LOGGER.debug("Radio element value: EDIT " + assessment.getAnonymousGrading());
		boolean isAnonymous = true;
		if (assessment.getAnonymousGrading().equals("N")) {
			isAnonymous = false;
		}
		LOGGER.debug("Radio element value is?? " + isAnonymous);
		
		boolean individualAssignmentType = true;
		if(assessment.getAssignmentType().equals("G")){
			individualAssignmentType = false;
		}
		
		boolean delegateGrading = false;
		if(assessment.getDelegateGrading().equals("Y")){
			delegateGrading = true;
		}
		
		boolean showStudGrading = false;
		if(assessment.getShow_stud_my_grade().equals("Y")){
			showStudGrading = true;
		}
		
		boolean showStatGrading = false;
		if(assessment.getShow_stat_my_grade().equals("Y")){
			showStatGrading = true;
		}
		
		boolean includeGradeCenterCalc = false;
		if(assessment.getInclude_grade_center_calc().equals("Y")){
			includeGradeCenterCalc = true;
		}
		
		boolean showEval = false;
		if(assessment.getShowEvalSub().equals("Y")){
			showEval = true;
		}
		
		boolean selfEval = false;
		if(assessment.getSelfEval().equals("Y")){
			selfEval = true;
		}
		
		boolean isReviewOfReview = true;
		if (assessment.getReviewOfReview().equals("N")) {
			isReviewOfReview = false;
		}
		
		//get the int possible points from the database and store it in the points possible variable
		int points_possible = assessment.getPointsPossible();
		
		//get string primary checked from database and store it in the variable
		Boolean[] primary_checkedList = {false, false, false, false, false};
		switch(assessment.getPrimaryCheck()){
		case("Score"):
			primary_checkedList[0] = true;
			break;
		case("Letter"):
			primary_checkedList[1] = true;
			break;
		case("Text"):
			primary_checkedList[2] = true;
			break;
		case("Percentage"):
			primary_checkedList[3] = true;
			break;
		case("Complete/Incomplete"):
			primary_checkedList[4] = true;
			break;
		
		}
		
		//get string primary checked from database and store it in the variable
		Boolean[] secondary_checkedList = {false, false, false, false, false};
		switch(assessment.getSecondaryCheck()){
		case("Score"):
			secondary_checkedList[0] = true;
			break;
		case("Letter"):
			secondary_checkedList[1] = true;
			break;
		case("Text"):
			secondary_checkedList[2] = true;
			break;
		case("Percentage"):
			secondary_checkedList[3] = true;
			break;
		case("Complete/Incomplete"):
			secondary_checkedList[4] = true;
			break;
		
		}
		
		Boolean[] grade_options = {false, false, false};
		switch(assessment.getGrade_options()){
		case("All possible graders"):
			secondary_checkedList[0] = true;
			break;
		case("All assigned to grade"):
			secondary_checkedList[1] = true;
			break;
		case("All unassigned graders"):
			secondary_checkedList[2] = true;
			break;
		
		}

		//Find stored files
		CourseFileManager crsFileMng = new CourseFileManager();
		mv.addObject("storedFiles", "No Files Stored");
		try {
			File root = crsFileMng.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE, course_id)));
			Path assessmentPath = Paths.get(root.getPath() + "/b2_peer_assessment/assessment_id_"
					+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, assessment_id)) + "/assessment_files");

			List<File> submittedfiles = new ArrayList<>();
			File folder = new File(assessmentPath.toString());
			for (File assessmentFile : folder.listFiles()) {
				LOGGER.debug("File in storage " + assessmentFile);
				submittedfiles.add(assessmentFile);
			}

			if (submittedfiles.size() > 0) {
				mv.addObject("storedFiles", submittedfiles);
			}
			//
		} catch (FileSystemException e) {
			LOGGER.debug("FAILED TO GET ROOT WHEN LOADING SUBMISSION: " + e);
			e.printStackTrace();
		}


		Rubric rubric = null;
		List<RubricRow> rubricRows = null;
		List<RubricColumn> rubricColumns = null;
		try {
			rubric = rubric_dao.findAllWithAssessmentID(Integer.parseInt(assessment_id)).get(0);
			rubricRows = rubric_row_dao.searchByRubricId(rubric.getPropertyId());
			rubricColumns = rubric_column_dao.searchByRubricId(rubric.getPropertyId());
		} catch (KeyNotFoundException e1) {
			e1.printStackTrace();
		} catch (NumberFormatException e1) {
			e1.printStackTrace();
		}
		rubricColumns.subList(0, rubric.getNumberOfScales() + 1);
		
		int counter = 0;
		ArrayList<ArrayList<ArrayList<String>>> cells = new ArrayList();
		ArrayList<ArrayList<String>> cellsPerRow = new ArrayList<>();
		for(int i = 0; i < rubricRows.size(); i++){
			for(int j = 0; j < rubricColumns.size(); j++){
				ArrayList<String> arrayCell = new ArrayList<>();
				RubricCell cell = rubric_cell_dao.searchByColAndRowId(rubricColumns.get(j).getPropertyId(), rubricRows.get(i).getPropertyId());
				arrayCell.add(cell.getPoints() + "");
				if(cell.getDescription() != null){
					arrayCell.add("\"" + cell.getDescription().replaceAll("(\\r|\\n|\\r\\n)+", "\\\\n").replaceAll("\"","\\\\\"") + "\"");
				}else{
					arrayCell.add("");
				}
				
				cellsPerRow.add(arrayCell);
				}
			cells.add(cellsPerRow);
			cellsPerRow = new ArrayList<>();
		
		}
		
		int max_points = rubric.getMaxPoints();
		

		// ModelAndView.addObject adds the objects you wish to pass to the jsp
		mv.addObject("assessment_id",assessment_id);
		mv.addObject("assessment_name", assessment_name);
		mv.addObject("assessment_instructions", assessment_instructions);
		mv.addObject("nr_of_submissions", nr_of_submissions);
		mv.addObject("date_range_assess_start_datetime", startSubDate);
		mv.addObject("date_range_assess_end_datetime", endSubDate);
		mv.addObject("date_range_peer_start_datetime", startPeerDate);
		mv.addObject("date_range_peer_end_datetime", endPeerDate);
		mv.addObject("isAnonymous", isAnonymous);
		mv.addObject("points_possible", points_possible);
		mv.addObject("primary_check", primary_checkedList);
		mv.addObject("secondary_check", secondary_checkedList);
		mv.addObject("delegate_grading", delegateGrading);
		mv.addObject("assignment_type",individualAssignmentType);
		mv.addObject("show_grade_options", grade_options);
		mv.addObject("show_stat_my_grade", showStatGrading);
		mv.addObject("show_stud_my_grade", showStudGrading);
		mv.addObject("include_grade_center_calc", includeGradeCenterCalc);
		mv.addObject("self_eval",selfEval);
		mv.addObject("show_eval_sub", showEval);
		mv.addObject("evaluation_instructions", evaluation_instructions);
		mv.addObject("isReviewOfReview", isReviewOfReview);
		mv.addObject("review_of_review_max_score", review_of_review_max_score);
		
		mv.addObject("rubric_max_points", max_points);
		mv.addObject("cellsInRows",cells);
		mv.addObject("numberOfScales", rubric.getNumberOfScales());
		
		LOGGER.debug("edit is now loaded!");

		// Returning the ModelAndView tells spring to find the JSP and set these
		// objects in the context for the request.
		return mv;
	}
	

	// Any time Spring receives a request for the URL properties, execute this
	// method. The method itself returns a ModelAndView. Spring uses this object
	// to locate the appropriate jsp file and
	// pass any variables you have set. RequestParam pulls the query parameter
	// from the request (ala request.getParameter("enabled")) amd sets it equal
	// to the variable you declare
	@UserAuthorization("system.admin.VIEW")
	@RequestMapping("/assessment/saveAssessment")
	private void saveAssessment(HttpServletRequest request, HttpServletResponse response, RedirectAttributes rattrs,
			@RequestParam(value ="assessment_id", required = false) Integer assessment_id, 
			@RequestParam("edit") boolean edit, 
			@RequestParam("course_id") String courseId,
			@RequestParam("content_id") String contentId,
			@RequestParam("name") String name, 
			@RequestParam("instructionstext") String instructions,
			@RequestParam("date_range_assess_start_datetime") String date_range_assess_start_datetime,
			@RequestParam("date_range_assess_end_datetime") String date_range_assess_end_datetime,
			@RequestParam("anon_eval") String anon_eval,
			@RequestParam("self_eval") String self_eval,
			@RequestParam("show_eval_sub") String show_eval_sub,
			@RequestParam("nr_of_submissions") int nr_of_submissions, 
			@RequestParam("user_id") String userId,
			@RequestParam("points_possible") int points_possible,
			@RequestParam("assignment_type") String assignment_type,
			@RequestParam(required = false, value="enable_delegate_grading") String enable_delegate_grading,
			@RequestParam(required = false, value="show_stat_my_grade") String show_stat_my_grade,
			@RequestParam(required = false, value="show_stud_my_grade") String show_stud_my_grade,
			@RequestParam(required = false, value="include_grade_center_calc") String include_grade_center_calc,
			@RequestParam("primary") String primary_check,
			@RequestParam("secondary") String secondary_check,
			@RequestParam("show_grade_options") String grade_options,
			@RequestParam("date_range_peer_start_datetime") String date_range_peer_start_datetime,
			@RequestParam("date_range_peer_end_datetime") String date_range_peer_end_datetime,
			@RequestParam("scale_points") String[] scalePoints,
			@RequestParam("numberOfScales") String numberOfScales,
			@RequestParam("criteria_name_input") String[] criteriaNames,
			@RequestParam("scale_descriptions") String[] criteriaDescriptions,
			@RequestParam("eval_instructionstext") String evaluation_instructions,
			@RequestParam("review_of_review") String review_of_review,
			@RequestParam("reivew_of_review_max_score") Integer reivew_of_review_max_score,
			@RequestParam(value="multiselect_right_values", required = false) String multiselect_right_values,
			@RequestParam(value="filePickerAddAssignmentInfo_LocalFile0", required = false) MultipartFile multipartFile
				
	) {

//		Enumeration<String> es = request.getParameterNames();
//		while(es.hasMoreElements()){
//			String esName = es.nextElement();
//			String values[] = request.getParameterValues(esName);
//			LOGGER.debug("submission Parametres: "+ esName + ", 	Value: " + values[0]);			
//		}
		
		Assessment assessment;
		if (edit) {
			assessment = assess_dao.loadById(assessment_id);
		} else {
			assessment = new Assessment();
		}

		// set all prefs to incoming request parameter values
		assessment.setName(name);
		assessment.setInstructions(instructions);
		assessment.setNrOfSubmissions(nr_of_submissions);
		assessment.setPointsPossible(points_possible);
		assessment.setReviewOfReviewMaxScore(reivew_of_review_max_score);
		assessment.setPrimaryCheck(primary_check);
		assessment.setSecondaryCheck(secondary_check);
		assessment.setGrade_options(grade_options);
		assessment.setEvaluationInstructions(evaluation_instructions);
		
		if(multiselect_right_values != null){
			assessment.setSubmissionGroups(multiselect_right_values);
		}else{
			assessment.setSubmissionGroups("");
		}
		
		if(show_stat_my_grade == null){
			assessment.setShow_stat_my_grade("N");
		}else{
			assessment.setShow_stat_my_grade("Y");
		}
		
		if(enable_delegate_grading == null){
			assessment.setDelegateGrading("N");
		}else{
			assessment.setDelegateGrading("Y");
		}
		
		if(include_grade_center_calc == null){
			assessment.setInclude_grade_center_calc("N");
		}else{
			assessment.setInclude_grade_center_calc("Y");
		}
		
		if(show_stud_my_grade == null){
			assessment.setShow_stud_my_grade("N");
		}else{
			assessment.setShow_stud_my_grade("Y");
		}
		
		//Set calendar dates. YYYY-MM-DD HH:MM:SS e.g 2017-04-09 12:00:00
		Calendar startDateCal = DatePickerUtil.pickerDatetimeStrToCal(date_range_assess_start_datetime);
		assessment.setSubmissionDateStart(startDateCal);
		Calendar endDateCal = DatePickerUtil.pickerDatetimeStrToCal(date_range_assess_end_datetime);
		assessment.setSubmissionDateEnd(endDateCal);
		
		Calendar startDateCalPeer = DatePickerUtil.pickerDatetimeStrToCal(date_range_peer_start_datetime);
		assessment.setPeerDateStart(startDateCalPeer);
		Calendar endDateCalPeer = DatePickerUtil.pickerDatetimeStrToCal(date_range_peer_end_datetime);
		assessment.setPeerDateEnd(endDateCalPeer);
		
		
		//Set radio button values
		if(anon_eval.equals("anon_no")){
			assessment.setAnonymousGrading("N");			
		}else{
			assessment.setAnonymousGrading("Y");
		}
	
		if(assignment_type.equals("individual_sub")){
			assessment.setAssignmentType("I");
		}else {
			assessment.setAssignmentType("G");
		}
		
		if(self_eval.equals("self_eval_yes")){
			assessment.setSelfEval("Y");			
		}else{
			assessment.setSelfEval("N");
		}
		
		if(show_eval_sub.equals("show_eval_yes")){
			assessment.setShowEvalSub("Y");			
		}else{
			assessment.setShowEvalSub("N");
		}
		
		if(review_of_review.equals("review_of_review_no")){
			assessment.setReviewOfReview("N");			
		}else{
			assessment.setReviewOfReview("Y");
		}

		Integer courseIdInt = null;
		Id courseIdParam = null;
		try {
			courseIdParam = Id.generateId(Course.DATA_TYPE, courseId);
			StringTokenizer sToken = new StringTokenizer(courseIdParam.toExternalString(), "_");

			courseIdInt = Integer.parseInt(sToken.nextToken());
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to save courseID!" + e1);
			e1.printStackTrace();
		}
		assessment.setCourseId(courseIdInt);

		// User id
		int userIdInt = 0;
		try {
			Id userIdParam = Id.generateId(User.DATA_TYPE, userId);
			StringTokenizer sToken = new StringTokenizer(userIdParam.toExternalString(), "_");

			userIdInt = Integer.parseInt(sToken.nextToken());
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		assessment.setUserId(userIdInt);

		// save the assessment
		assess_dao.save(assessment);
		
		Assessment newAssessment = assess_dao.loadLast();
		int newAssessmentID = newAssessment.getPropertyId();

		Rubric rubric = null;
		if(edit){
			try {
				rubric = rubric_dao.findAllWithAssessmentID(assessment_id).get(0);
			} catch (KeyNotFoundException e) {
				e.printStackTrace();
			}
		}else {
			rubric = new Rubric();
		}

		int counter = 1;
		int max_points_row = 0;
		int total_points = 0;
		for (String point : scalePoints){
			if(counter == Integer.parseInt(numberOfScales)){
				total_points += max_points_row;
				counter = 1;
				max_points_row = 0;
			}else {
				counter++;
				int intPoint = Integer.parseInt(point);
				if(intPoint > max_points_row){
					max_points_row = intPoint;
				}
				
			}
			
		}
		
		rubric.setMaxPoints(total_points);
		rubric.setPeerAssessmentId(newAssessmentID);
		rubric.setNumberOfScales(Integer.parseInt(numberOfScales));
		
		rubric_dao.save(rubric);
		
		int newRubricID = rubric_dao.loadLast().getPropertyId();
		int cellIndex = 0;
		
		for(int i = 0; i < criteriaNames.length; i++){
			RubricRow rubricRow = null;
			if(edit){
				try {
					LOGGER.debug("rowerr " + i+1);
					rubricRow = rubric_row_dao.searchByPositionAndId(i+1, newRubricID);
					if(rubricRow == null){
						rubricRow = new RubricRow();
					}
				} catch (KeyNotFoundException e) {
					e.printStackTrace();
					rubricRow = new RubricRow();
				}
				
			}else{
				rubricRow = new RubricRow();
			}
			rubricRow.setHeader(criteriaNames[i]);
			rubricRow.setPosition(i+1);
			rubricRow.setRubricId(newRubricID);
			rubric_row_dao.save(rubricRow);
			for(int j = 0; j < (Integer.parseInt(numberOfScales) + 1); j++){
				if(i == 0){
					RubricColumn rubricColumn = null;
					if(!edit){
						rubricColumn = new RubricColumn();
					}else {
						try {
							rubricColumn = rubric_column_dao.searchByPositionAndId(j, newRubricID);
							if(rubricColumn == null){
								rubricColumn = new RubricColumn();
							}
						} catch (KeyNotFoundException e) {
							e.printStackTrace();
							rubricColumn = new RubricColumn();
						}
					}
					rubricColumn.setPosition(j);
					rubricColumn.setRubricId(newRubricID);
					rubric_column_dao.save(rubricColumn);
				}
				
				int rubricColumnID;
				int rubricRowID;
				try {
					
					rubricColumnID = rubric_column_dao.searchByPositionAndId(j,newRubricID).getPropertyId();
					
					rubricRowID = rubric_row_dao.searchByPositionAndId(i+1,newRubricID).getPropertyId();
					
					RubricCell rubricCell = null;
					if(!edit){
						rubricCell = new RubricCell();
					}else{
						rubricCell = rubric_cell_dao.searchByColAndRowId(rubricColumnID, rubricRowID);
						if(rubricCell == null){
							rubricCell = new RubricCell();
						}
					}
					rubricCell.setRubricRowId(rubricRowID);
					rubricCell.setRubricColumnId(rubricColumnID);
					if(j == 0){
						rubricCell.setDescription(criteriaNames[i]);
						rubricCell.setPoints(0);
					}else {
						cellIndex = i*Integer.parseInt(numberOfScales) + j - 1;
						int points = Integer.parseInt(scalePoints[cellIndex]);
						String description = criteriaDescriptions[cellIndex];
						LOGGER.debug("cellIndex " + cellIndex +  " newpoints1 " + points + " description " + description);
						rubricCell.setPoints(points);
						if(description == null){
							rubricCell.setDescription(" ");
						}else{
							rubricCell.setDescription(description);
						}
						
					}
					
					rubric_cell_dao.save(rubricCell);
				} catch (KeyNotFoundException e) {
					e.printStackTrace();
				}
				
			}
		}
		
		//Evaluator pairing
		if(!edit){
			Id courseID_users;
			try {
				courseID_users = Id.generateId( Course.DATA_TYPE, courseId );
				List<User> usersInCourse = _userLoader.loadByCourseId(courseID_users);
				String type = assignment_type;
				Integer evaluation_number = nr_of_submissions;
				int pairingsSet = 0;
				long seed = System.nanoTime();
				HashMap<Id,ArrayList> submission_evaluations = new HashMap<>();
				HashMap<Id,ArrayList> evaluator_submissions = new HashMap<>();
				Collections.shuffle(usersInCourse, new Random(seed));
				for(User user : usersInCourse){
					evaluator_submissions.put(user.getId(), new ArrayList<>());
					submission_evaluations.put(user.getId(), new ArrayList<>());
				}
				//Get groups
				List<Group> groupList = new ArrayList<Group>();
				try {
					groupList = _groupLoader.loadByCourseId(courseID_users);
				} catch (KeyNotFoundException e) {
					e.printStackTrace();
				} catch (PersistenceException e) {
					e.printStackTrace();
				}
				
				if(type.equals("individual_sub")){
					for(int i = 0; i < usersInCourse.size(); i++){
						User user = usersInCourse.get(i);
						if(assessment.getSelfEval().equals("Y")){
							student_review_dao.save(new StudentReview(getUserIdAsInt(user.getId().toString()), getUserIdAsInt(user.getId().toString()), newAssessmentID, "N"));
						}
						for(int y = 0; y < nr_of_submissions; y++){
							if(y == usersInCourse.size()){
								break;
							}
							User userToEvaluate = findUserToEvaluate(user,submission_evaluations, nr_of_submissions);
							ArrayList newListEvaluator = submission_evaluations.get(userToEvaluate.getId());
							newListEvaluator.add(user);
							submission_evaluations.put(userToEvaluate.getId(), newListEvaluator);
	
							student_review_dao.save(new StudentReview(getUserIdAsInt(userToEvaluate.getId().toString()), getUserIdAsInt(user.getId().toString()), newAssessmentID, "N"));
						}
						LOGGER.debug("Number of evals: " + student_review_dao.searchByUserIDAndAssessmentID(getUserIdAsInt(user.getId().toString()),newAssessmentID).size());
					}
					
				}else {
//					for(int i = 0; i < groupList.size(); i++){
//						Group group = groupList.get(i);
//						if(assessment.getSelfEval().equals("Y")){
//							student_review_dao.save(new StudentReview(getDataTypeIdAsInt(Group.DATA_TYPE, group.getId().toString()), getDataTypeIdAsInt(Group.DATA_TYPE, group.getId().toString()), newAssessmentID, "N"));
//						}
//						for(int y = 0; y < groupList.size(); y++){
//							if(y == groupList.size()){
//								break;
//							}
//							//TODO: not fully implemented. Selects all groups besides you
//							if(!groupList.get(y).equals(group)){
//								Group groupToEvaluate = groupList.get(y);
//								student_review_dao.save(new StudentReview(getDataTypeIdAsInt(Group.DATA_TYPE, groupToEvaluate.getId().toString()), getDataTypeIdAsInt(Group.DATA_TYPE, group.getId().toString()), newAssessmentID, "N"));
//							}
//						}
//						LOGGER.debug("Number of evals: " + student_review_dao.searchByUserIDAndAssessmentID(getUserIdAsInt(group.getId().toString()),newAssessmentID).size());
//					}
				}
				
			} catch (PersistenceException e3) {
				e3.printStackTrace();
			}
		
		}
		
		/**
		 * Adding the assessment to the course content list
		 * https://github.com/shurrey/BBDN-LTI-API-Sample/blob/master/src/main/webapp/contentHandlers/create_proc.jsp
		 */
		Content courseDoc = null;
		 try {
			 Id courseID = Id.generateId( Course.DATA_TYPE, courseId );
			 Id folderID = Id.generateId( Content.DATA_TYPE, contentId );
			  
			ContentDbLoader loader = ContentDbLoader.Default.getInstance();
			ContentDbPersister persister= ContentDbPersister.Default.getInstance();
			
			

			if (edit) {
				courseDoc  = loader.loadById(folderID);
			} else {
				courseDoc = new Content();
				courseDoc.setContentHandler("resource/x-ntnu-peer-review");
				courseDoc.setCourseId( courseID );
				courseDoc.setParentId( folderID );
			}
			courseDoc.setTitle( name );
			
			 ExtendedData ed = new ExtendedData();
			 
			 ed.setValue("contentType", "assessment");
			 ed.setValue(ASSESSMENT_ID, newAssessmentID +"");
//			 ed.setValue("customParameter2",mr.getParameter( "customParameter2"));
//			 ed.setValue("customParameter3",mr.getParameter( "customParameter3"));
//			 ed.setValue("customParameter4",mr.getParameter( "customParameter4"));
			 courseDoc.setExtendedData(ed);
	
			 courseDoc.setIsAvailable(true);
			 courseDoc.validate();
			 persister.persist( courseDoc );
			
			 newAssessment.setContentId(getDataTypeIdAsInt(Content.DATA_TYPE, courseDoc.getId().getExternalString()));
			 assess_dao.save(newAssessment);
			 
			 String editLink = PlugInUtil.getUri("ntnu", "b2_peer_review_tool", "assessment/edit?course_id="+assessment.getCourseId()+"&content_id=" + courseDoc.getId().getExternalString());
			 String submitLink = PlugInUtil.getUri("ntnu", "b2_peer_review_tool", "submission/preSubmit?course_id="+assessment.getCourseId()+"&content_id=" + courseDoc.getId().getExternalString());
			 String evaluatorSubLink = PlugInUtil.getUri("ntnu", "b2_peer_review_tool", "review/student_reviews?course_id="+assessment.getCourseId()+"&assessment_id="+newAssessmentID+"&content_id=" + courseDoc.getId().getExternalString());
			 String instructorSubLink = PlugInUtil.getUri("ntnu", "b2_peer_review_tool", "review/instructor_reviews?course_id="+assessment.getCourseId()+"&assessment_id="+newAssessmentID+"&content_id="+ courseDoc.getId().getExternalString());
			 FormattedText bltiDescription = new FormattedText(instructions +"<br>"+
					"<h3><a style=color:rgb(0,80,158) href=\"" + submitLink + "\">Submission</a></h3>"+
					"<a href=\"" + evaluatorSubLink + "\">Evaluate Submissions</a> <br>"+
//        			"<a href=\"" + instructorSubLink + "\">Instructor Submission</a>"+
        			"<p> Submission dates: "+assessment.getStartDateAsString() +" to "+ assessment.getEndDateAsString() +"<br>"+
        			"Evaluation dates: "+assessment.getEvaluationStartDateAsString() +" to "+ assessment.getEvaluationEndDateAsString() +"</p>"
        			,FormattedText.Type.SMART_TEXT);
			 
			  courseDoc.setBody( bltiDescription );
			  persister.persist( courseDoc );
			  
		} catch (PersistenceException e1) {
			LOGGER.debug("Content loader or persister .getInstance failed or Id generation failed");
			e1.printStackTrace();
		} catch (ValidationException e) {
			LOGGER.debug("Validation failed");
			e.printStackTrace();
		}

		 //Creates a gradable item for this assessment
		 GradableItem gi = null;
			GradableItemManager giManager = GradebookManagerFactory.getGradableItemManager();
			GradeManager tt = GradebookManagerFactory.getGradeManager();
		 
			try {
				gi = giManager.getGradebookItemByContentId(courseDoc.getId());
			} catch (PersistenceRuntimeException e1) {
				LOGGER.debug("Failed to get gradable item: " + e1);
				e1.printStackTrace();
			} catch (BbSecurityException e1) {
				LOGGER.debug("Failed to get gradable item: " + e1);
				e1.printStackTrace();
			}
			if (gi == null) {
				gi = new GradableItem();
				LOGGER.debug("No gradeable item found, creating a new one: " + gi);
			}
			
			LOGGER.debug("Course ID: " + courseId + " Content ID: " + courseDoc.getId().getExternalString() + " Is Manual: " + gi.isManual());
			gi.setCourseId(courseIdParam);
			gi.setCourseContentId(courseDoc.getId());
			gi.setDisplayTitle(assessment.getName());
			gi.setPoints(assessment.getPointsPossible());
			gi.setTitle(assessment.getName());
			gi.setVisibleInBook(true);
			gi.setVisibleToStudents(true);
			

			try {
				giManager.persistGradebookItem(gi);
			} catch (BbSecurityException e) {
				LOGGER.debug("Failed to persist gradable item: " + e);
				e.printStackTrace();
			}
			
			/**
			 * This whole try method uses the inner file structure of a course to store the submitted files.
			 * This might not be a good idea for a b2 in prodution but it works well for testing purposes.
			 * There is also a permission in the manifest connected to this "try" code, needed for writing/reading the file.
			 */
			//TODO: Make it so that the file is stored by name inside a folder with user id (e.g pk1_"id"/ "fileHere") instead of todays pk1_"id" as the name of the file
			CourseFileManager crsFileMng = new CourseFileManager();
			
			try {
				
				LOGGER.error("(Assessment) TRYING TO CREATE FOLDER STRUCTURE with Course ID: " + Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE,courseId)));
				File root = crsFileMng.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE,courseId)));
				
				//If the root folder does not exsists
				if(!root.exists()){
					boolean succsess = root.mkdirs(); //TODO: should have its own method that returns bool and if returning false should give error
					LOGGER.debug("(Assessment) ROOT DID NOT EXSIST, CREATING ROOT. Is creating root a success? " + succsess);
				}
				
				//If the default peer review folders does not exsists it creates it
				Path path = Paths.get(root.getPath()+"/b2_peer_assessment/assessment_id_"+Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE,Integer.toString(newAssessmentID)))+"/submissions_files");
				if(!Files.exists(path)){
					 try {
					      Files.createDirectories(path);
					    } catch (IOException e) {
					    LOGGER.debug("(Assessment) FAILED TO CREATE B2 DIRECTORIES IN ROOT for submission_files: "+ e);
					      e.printStackTrace();
					    }
				}
				
				//If the default assessment files folder does not exsists it creates it.
				Path assessmentPath = Paths.get(root.getPath()+"/b2_peer_assessment/assessment_id_"+Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE,Integer.toString(newAssessmentID)))+"/assessment_files");
				if(!Files.exists(assessmentPath)){
					 try {
					      Files.createDirectories(assessmentPath);
					    } catch (IOException e) {
					    LOGGER.debug("(Assessment) FAILED TO CREATE B2 DIRECTORIES IN ROOT for assessment_files: "+ e);
					      e.printStackTrace();
					    }
				}
				
				//Debugging output to display all files inside our submission_files folder:
//				File folder = new File(path.toString()+"/");
				File folder = new File(root.getPath()+"/b2_peer_assessment/assessment_id_"+Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE,Integer.toString(newAssessmentID)))+"/submissions_files/");
				File[] listOfFiles = folder.listFiles();
				LOGGER.debug("(Assessment) Running List of files on "+ path + "/" + " RESULT: " + listOfFiles + " With Length: " + listOfFiles.length);
				    for (int i = 0; i < listOfFiles.length; i++) {
				      if (listOfFiles[i].isFile()) {
				    	  LOGGER.debug("File "+ listOfFiles[i].getName());
				      } else if (listOfFiles[i].isDirectory()) {
				    	  LOGGER.debug("Directory " + listOfFiles[i].getName());
				      }
				    }
			} catch (FileSystemException e2) {
				LOGGER.debug("(Assessment) FileSystemException" + e2);
				e2.printStackTrace();
			} 
			
		/**
		 * Saving assessment files
		 */
		if (multipartFile != null) {
			try {

				File file = convert(multipartFile);
				File root = crsFileMng
						.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE, courseId)));

				Path path = Paths.get(root.getPath() + "/b2_peer_assessment/assessment_id_"
						+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, Integer.toString(newAssessmentID)))
						+ "/assessment_files");

				try {
					Path fileToMovePath = Paths.get(file.getAbsolutePath());
					Path targetPath = Paths.get(path + "/" + file.getName());
					Files.move(fileToMovePath, targetPath, StandardCopyOption.REPLACE_EXISTING);
				} catch (IOException e) {
					LOGGER.debug("FAILED TO MOVE FILE: " + e);
					e.printStackTrace();
				}

				// Debugging output to display all files inside our submission_files folder:
				// File folder = new File(path.toString()+"/");
				File folder = new File(root.getPath() + "/b2_peer_assessment/assessment_id_"
						+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, Integer.toString(newAssessmentID)))
						+ "/assessment_files/");
				File[] listOfFiles = folder.listFiles();
				LOGGER.debug("Running List of assessment files on " + path + "/" + " RESULT: " + listOfFiles + " With Length: "
						+ listOfFiles.length);
				for (int i = 0; i < listOfFiles.length; i++) {
					if (listOfFiles[i].isFile()) {
						LOGGER.debug("File " + listOfFiles[i].getName());
					} else if (listOfFiles[i].isDirectory()) {
						LOGGER.debug("Directory " + listOfFiles[i].getName());
					}
				}

			} catch (FileSystemException e2) {
				e2.printStackTrace();
			} catch (IOException e1) {
				LOGGER.error("FAILED TO CONVERT MULTIPART FILE");
				e1.printStackTrace();
			} catch (NullPointerException e1) {
				LOGGER.error("NO FILE FOUND");
				e1.printStackTrace();
			}
		}
			
		try {
			ReceiptOptions ro = new ReceiptOptions();
			if(edit){
        		ro.addSuccessMessage("Assessment Successfully Updated.");
        	}else{
        		ro.addSuccessMessage("Assessment Successfully Created.");
        	}
			InlineReceiptUtil.addReceiptToRequest(request, ro);

			final Context context = ContextManagerFactory.getInstance().getContext();  
			final String contentReturnURL = PlugInUtil.getContentReturnURL(context.getContentId(), context.getCourseId());    
			
			response.sendRedirect(contentReturnURL);  
	
		} catch (IOException e) {
			e.printStackTrace();
		} 
	}
	
	/**
	 * Helper method for making UserID an int when you got string
	 * @param userId
	 * @return
	 */
	private int getUserIdAsInt(String userId){
		int userIdInt = 0;
		try {
			Id userIdParam = Id.generateId(User.DATA_TYPE, userId);
			StringTokenizer sToken = new StringTokenizer(userIdParam.toExternalString(), "_");
			userIdInt = Integer.parseInt(sToken.nextToken());
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		return userIdInt;
	}
	
	private User findUserToEvaluate(User user ,HashMap<Id,ArrayList> submission_evaluations, int numberOfSubs) throws KeyNotFoundException, PersistenceException{
		Iterator it = submission_evaluations.entrySet().iterator();
		Id potential_eval = null;
		int size = numberOfSubs + 5;
		boolean variate = false;
	    while (it.hasNext()) {
	        Map.Entry pair = (Map.Entry)it.next();
 
	        if((Id)pair.getKey() != user.getId() && !submission_evaluations.get((Id)pair.getKey()).contains(user)){
	        	if(variate){
	        		if(((ArrayList) pair.getValue()).size() == 0){
			        	return _userLoader.loadById((Id)pair.getKey());
			        }else if(((ArrayList) pair.getValue()).size() <= size){
			        	size = ((ArrayList) pair.getValue()).size();
			        	potential_eval = (Id)pair.getKey();
			        }
	        	}else {
	        		if(((ArrayList) pair.getValue()).size() == 0){
			        	return _userLoader.loadById((Id)pair.getKey());
			        }else if(((ArrayList) pair.getValue()).size() < size){
			        	size = ((ArrayList) pair.getValue()).size();
			        	potential_eval = (Id)pair.getKey();
			        }
	        	}
	        	variate = !variate;
		        
	        }
	    }
	    LOGGER.debug("Potential eval: " + potential_eval);
		return _userLoader.loadById(potential_eval);
	}
	private int getDataTypeIdAsInt(DataType dataType, String id){
		int idInt = 0;
		try {
			Id idParam = Id.generateId(dataType, id);
			StringTokenizer sToken = new StringTokenizer(idParam.toExternalString(), "_");
			idInt = Integer.parseInt(sToken.nextToken());
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		return idInt;
	}
	
	private String getCourseContentID(DataType dataType, String id){
		String idAsString = "_";
		try {
			Id idParam = Id.generateId(dataType, id);
			StringTokenizer sToken = new StringTokenizer(idParam.toExternalString(), "_");
			idAsString += sToken.nextToken();
			idAsString += "_";
			idAsString += sToken.nextToken();
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		return idAsString;
	}
	public File convert(MultipartFile file) throws IOException
	{    
	    File convFile = new File(file.getOriginalFilename());
	    convFile.createNewFile(); 
	    FileOutputStream fos = new FileOutputStream(convFile); 
	    fos.write(file.getBytes());
	    fos.close(); 
	    return convFile;
	}
	
}
